#include "frame_preview.h"



class PreviewCanvas : public wxGLCanvas
{
friend class PreviewFrame;
public:
	PreviewCanvas(wxWindow *parent, const wxPoint& = wxDefaultPosition, const wxSize& = wxDefaultSize);
	PreviewCanvas(wxWindow *parent, const PreviewCanvas& other,
		const wxPoint& = wxDefaultPosition, const wxSize& = wxDefaultSize);
	void Init();
	void Render();
	void OnPaint(wxPaintEvent& event);
	void OnSize(wxSizeEvent& event);
	void OnKeyDown(wxKeyEvent& event);
	void OnKeyUp(wxKeyEvent& event);
	void OnMouse(wxMouseEvent& event);
	void OnEnterWindow( wxMouseEvent& event );

private:
	int display_mode;
	gl::Camera camera;
	gl::Light light;
	vector2 mouse_pos, pre_mouse_pos;
	vector4 pre_camera_pos;

	DECLARE_EVENT_TABLE()
};





enum
{
	MENU_SELECT_MODEL= 100,
	MENU_SELECT_TEXTURE,
};


BEGIN_EVENT_TABLE( PreviewFrame, wxFrame )
	EVT_PAINT( PreviewFrame::OnPaint )
	EVT_MENU( MENU_SELECT_MODEL, PreviewFrame::OnSelectModel )
	EVT_MENU( MENU_SELECT_TEXTURE, PreviewFrame::OnSelectTexture )
END_EVENT_TABLE()


PreviewFrame::PreviewFrame(wxWindow *parent) :
	wxFrame(parent, -1, wxT("preview"))
{
	wxMenu *menu_file = new wxMenu();
	menu_file->Append(MENU_SELECT_MODEL, wxT("Model"));
	menu_file->Append(MENU_SELECT_TEXTURE, wxT("Texture"));
//	menu_file->AppendSeparator();

	wxMenuBar *menu_bar = new wxMenuBar();
	menu_bar->Append(menu_file, wxT("&File"));
	SetMenuBar(menu_bar);


	canvas = new PreviewCanvas(this, wxPoint(0, 0), wxSize(250, 250));
	wxBoxSizer *bsizer = new wxBoxSizer(wxVERTICAL);
	bsizer->Add( canvas, 1, wxEXPAND );
	SetAutoLayout( TRUE );
	SetSizer( bsizer );


	CreateStatusBar();
}

void PreviewFrame::OnPaint(wxPaintEvent& event)
{
	canvas->Render();
	event.Skip();
}

void PreviewFrame::OnSelectModel(wxCommandEvent& event)
{
	wxFileDialog file_dlg(this, wxT("open model"), wxT(""), wxT(""),
		wxT("*.mqo;*.obj)") );

	if(file_dlg.ShowModal() == wxID_OK) {
		canvas->SetCurrent();
		bool result = GetModel().load( file_dlg.GetPath().c_str() );
		if(result)
			SetStatusText( wxT("load success") );
		else
			SetStatusText( wxT("load failed") );

		Refresh();
	}
}

void PreviewFrame::OnSelectTexture(wxCommandEvent& event)
{
	wxFileDialog file_dlg(this, wxT("open texture"), wxT(""), wxT(""),
		wxT("*.bmp;*.tga") );

	if(file_dlg.ShowModal() == wxID_OK) {
		canvas->SetCurrent();
		bool result = GetTexture().load( file_dlg.GetPath().c_str() );
		if(result)
			SetStatusText( wxT("load success") );
		else
			SetStatusText( wxT("load failed") );

		Refresh();
	}
}




BEGIN_EVENT_TABLE( PreviewCanvas, wxGLCanvas )
	EVT_SIZE( PreviewCanvas::OnSize )
	EVT_PAINT( PreviewCanvas::OnPaint )
	EVT_KEY_DOWN( PreviewCanvas::OnKeyDown )
	EVT_KEY_UP( PreviewCanvas::OnKeyUp )
	EVT_MOUSE_EVENTS( PreviewCanvas::OnMouse )
	EVT_ENTER_WINDOW( PreviewCanvas::OnEnterWindow )
END_EVENT_TABLE()

PreviewCanvas::PreviewCanvas(wxWindow *parent, const wxPoint& pos, const wxSize& size) :
	wxGLCanvas(parent, -1, pos, size)
{
	Init();
}

PreviewCanvas::PreviewCanvas(wxWindow *parent, const PreviewCanvas& other, const wxPoint& pos, const wxSize& size) :
	wxGLCanvas(parent, other.GetContext(), -1, pos, size )
{
	Init();
}


void PreviewCanvas::Init()
{
	display_mode = 0;
	camera.position = vector4(0, 360, 1);
	pre_camera_pos = camera.position;
	light.position = vector4(200, 300, -200);
	camera.target = vector4(0, 0, 0);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glShadeModel( GL_SMOOTH );
	glCullFace( GL_BACK );
	glFrontFace( GL_CCW );
	glEnable( GL_CULL_FACE );
}

void PreviewCanvas::OnPaint(wxPaintEvent& event)
{
	Render();
}

void PreviewCanvas::OnSize(wxSizeEvent& event)
{
	wxGLCanvas::OnSize(event);

	int w, h;
	GetClientSize(&w, &h);
	SetCurrent();
	glViewport(0, 0, w, h);

	Render();
}

void PreviewCanvas::OnKeyDown(wxKeyEvent& event)
{
	Render();
	event.Skip();
}

void PreviewCanvas::OnKeyUp(wxKeyEvent& event)
{
	Render();
	event.Skip();
}

void PreviewCanvas::OnMouse(wxMouseEvent& event)
{
	if(!event.Moving()) {
		return;
	}

	bool redraw_flag = false;

	mouse_pos.x = event.GetX();
	mouse_pos.y = event.GetY();

	vector2 progress = mouse_pos - pre_mouse_pos;

	if(event.LeftDown()) {
		pre_mouse_pos = mouse_pos;
		pre_camera_pos= camera.position;
	}
	else if(event.LeftIsDown()) {
		static const matrix33 turn90degree = matrix33().rotateY(90);
		float rotate_speed = 15;
		float ang_xz = progress.y*rotate_speed*radian;
		float ang_y = -progress.x*rotate_speed*radian;
		vector3 axis = (turn90degree*vector3(camera.position.x, 0, camera.position.z)).normalize();

		matrix33 camera_rot;
		camera_rot.rotateA(axis, ang_xz);
		camera_rot.rotateY(ang_y);

		camera.position = camera_rot*camera.position;
		redraw_flag = true;
	}

	if(event.MiddleIsDown()) {
	}

	if(event.RightIsDown()) {
		camera.position *= (1+(progress.x-progress.y)*0.003);
		redraw_flag = true;
	}

	pre_mouse_pos = mouse_pos;
	pre_camera_pos= camera.position;

	if(redraw_flag)
		Render();
}

void PreviewCanvas::OnEnterWindow( wxMouseEvent& event )
{
	SetFocus();
}


void PreviewCanvas::Render()
{
	wxPaintDC dc(this);
	SetCurrent();


	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	int w, h;
	GetClientSize(&w, &h);

	gluPerspective( 60.0, static_cast<float>(w)/static_cast<float>(h), 1.0, 1024.0 );
	glMatrixMode( GL_MODELVIEW );

	camera.look();

	glEnable(GL_BLEND);
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	light.enable();
	GetTexture().assign();
	{
		gl::MatrixSaver msaver;
		matrix44 matrix;
		vector4 model_pos;
		if(!GetTimeline3Map().empty())
			model_pos = vector4(GetTimeline3().getValue(GetCurrentFrame()), 1);
		matrix.translate(model_pos.v);
		glMultMatrixf(matrix.v);

		GetModel().draw();
	}

	glDisable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);


	SwapBuffers();
}
