#include "frame_timeline.h"
#include "frame_preview.h"





class TimelineCanvas : public wxGLCanvas
{
friend class TimelineFrame;
public:
	TimelineCanvas(wxWindow *parent, const wxPoint& = wxDefaultPosition, const wxSize& = wxDefaultSize);
	void Init();
	void Render();
	void AdjustTimeSlider();
	void OnPaint(wxPaintEvent& event);
	void OnSize(wxSizeEvent& event);
	void OnKeyDown(wxKeyEvent& event);
	void OnKeyUp(wxKeyEvent& event);
	void OnMouse(wxMouseEvent& event);
	void OnMouseButton(wxMouseEvent& event);
	void OnMouseMove(wxMouseEvent& event);
	void OnEnterWindow( wxMouseEvent& event );

private:
	vector2 mouse_pos;
	vector2 pre_mouse_pos;
	int edit_mode;
	float screen_region[4];
	bool toggle_help;
	Timeline1::iterator focus_point;
	Timeline1 *focus_tl1;

	enum {
		EM_EDIT=0,
		EM_INSERT=1,
		EM_DELETE=2,
	};

	DECLARE_EVENT_TABLE()
};






void RedrawAll()
{
	GetPreviewFrame()->Refresh();
	GetTimelineFrame()->Refresh();
}






enum {
	MENU_LOAD_TIMELINE= 100,
	MENU_SAVE_TIMELINE,
	TEXTCTRL_TIMELINE,
	SLIDER_TIMELINE,
	CHOICE_TIMELINE,
	RADIOBOX_EDITMODE,
	BUTTON_NEW,
	BUTTON_COPY,
	BUTTON_DELETE,
};

BEGIN_EVENT_TABLE( TimelineFrame, wxFrame )
	EVT_PAINT( TimelineFrame::OnPaint )
	EVT_MENU( MENU_LOAD_TIMELINE, TimelineFrame::OnLoadTimeline )
	EVT_MENU( MENU_SAVE_TIMELINE, TimelineFrame::OnSaveTimeline )
	EVT_TEXT_ENTER( TEXTCTRL_TIMELINE, TimelineFrame::OnTextChanged )
	EVT_COMMAND_SCROLL( SLIDER_TIMELINE, TimelineFrame::OnSliderChanged )
	EVT_CHOICE( CHOICE_TIMELINE, TimelineFrame::OnChoiceSelected )
	EVT_BUTTON( BUTTON_NEW, TimelineFrame::OnNewButton )
	EVT_BUTTON( BUTTON_COPY, TimelineFrame::OnCopyButton )
	EVT_BUTTON( BUTTON_DELETE, TimelineFrame::OnDeleteButton )
	EVT_RADIOBOX( RADIOBOX_EDITMODE, TimelineFrame::OnEditModeSelected )
END_EVENT_TABLE()

TimelineFrame::TimelineFrame(wxWindow *parent) :
	wxFrame(parent, -1, wxT("timeline"))
{
	wxMenu *menu_file = new wxMenu();
	menu_file->Append(MENU_LOAD_TIMELINE, wxT("Load"));
	menu_file->Append(MENU_SAVE_TIMELINE, wxT("Save"));

	wxMenuBar *menu_bar = new wxMenuBar();
	menu_bar->Append(menu_file, wxT("&File"));
	SetMenuBar(menu_bar);


	CreateStatusBar(2);


	wxBoxSizer *top_sizer = new wxBoxSizer(wxHORIZONTAL);

	{
		wxPanel *slider_panel = new wxPanel( this, -1, wxPoint(0, 0), wxSize(400, 33) );
		wxBoxSizer *slider_sizer = new wxBoxSizer(wxHORIZONTAL);
		timeline_slider = new wxSlider( slider_panel, SLIDER_TIMELINE, 0, 0, 100 );
		timeline_text = new wxTextCtrl( slider_panel, TEXTCTRL_TIMELINE, wxT("0"), wxDefaultPosition, wxSize(40, -1) );
		slider_sizer->Add( timeline_slider, 1, wxEXPAND );
		slider_sizer->Add( timeline_text, 0, wxALIGN_RIGHT );
		slider_panel->SetAutoLayout( TRUE );
		slider_panel->SetSizer( slider_sizer );

		canvas = new TimelineCanvas(this, wxPoint(0, 0), wxSize(400, 300));

		wxBoxSizer *left_sizer = new wxBoxSizer(wxVERTICAL);
		left_sizer->Add( canvas, 1, wxEXPAND );
		left_sizer->Add( slider_panel, 0, wxEXPAND );

		top_sizer->Add( left_sizer, 1, wxEXPAND);
	}

	{
		wxPanel *right_panel = new wxPanel( this, -1, wxPoint(0, 0), wxSize(120, 30), wxSUNKEN_BORDER );
		wxBoxSizer *right_sizer = new wxBoxSizer(wxVERTICAL);
		timeline_choice = new wxChoice( right_panel, CHOICE_TIMELINE);

		wxButton *button1 = new wxButton( right_panel, BUTTON_NEW, wxT("new") );
		wxButton *button2 = new wxButton( right_panel, BUTTON_COPY, wxT("copy") );
		wxButton *button3 = new wxButton( right_panel, BUTTON_DELETE, wxT("delete") );

		wxString button_labels[3] = {
			wxT("edit"),
			wxT("insert"),
			wxT("delete"),
		};

		mode_selector = new wxRadioBox( right_panel, RADIOBOX_EDITMODE, wxT("edit mode"),
			wxDefaultPosition, wxDefaultSize, 3, button_labels, 1, wxRA_SPECIFY_COLS);

		right_sizer->Add( timeline_choice, 0, wxEXPAND | wxALL, 5 );
		right_sizer->Add( button1, 0, wxEXPAND | wxALL, 5 );
		right_sizer->Add( button2, 0, wxEXPAND | wxALL, 5 );
		right_sizer->Add( button3, 0, wxEXPAND | wxALL, 5 );
		right_sizer->Add( mode_selector, 0, wxEXPAND | wxALL, 5 );
		right_panel->SetAutoLayout( TRUE );
		right_panel->SetSizer( right_sizer );

		top_sizer->Add( right_panel, 0, wxEXPAND);
	}


	SetAutoLayout( TRUE );
	SetSizer( top_sizer );
}

void TimelineFrame::OnPaint(wxPaintEvent& event)
{
	canvas->Render();
	event.Skip();
}

void TimelineFrame::OnLoadTimeline(wxCommandEvent&)
{
	wxFileDialog file_dlg(this, wxT("load timeline"), wxT(""), wxT(""), wxT("lua script (*.lua)|*.lua"));

	if(file_dlg.ShowModal() == wxID_OK) {
		GetTimeline1Map().clear();
		GetTimeline3Map().clear();
		timeline_choice->Clear();
		canvas->focus_point = Timeline1::iterator();
		canvas->focus_tl1=0;

		lua::Thread lthread;
		timeline::regist_timeline_functions(lthread);

		if(lua_dofile(lthread, file_dlg.GetPath().c_str())==0) {
			timeline3_map& tl3_map = GetTimeline3Map();

			if(!tl3_map.empty()) {
				GetCurrentTimeline3Name() = tl3_map.begin()->first;
				canvas->AdjustTimeSlider();
				RedrawAll();
				RefreshChoice();
			}
		}

	}
}



char* create_point_string(Timeline1::point_t& p)
{
	static char buf[512];
	sprintf(buf, "vector2.new(%.2f, %.2f), vector2.new(%.2f, %.2f), vector2.new(%.2f, %.2f)",
		p.in.x, p.in.y,  p.pos.x, p.pos.y,  p.out.x, p.out.y);

	return buf;
}

void TimelineFrame::OnSaveTimeline(wxCommandEvent&)
{
	wxFileDialog file_dlg(this, wxT("save timeline"), wxT(""), wxT(""), wxT("lua script (*.lua)|*.lua"));

	if(file_dlg.ShowModal() == wxID_OK) {
		FILE *out = fopen(file_dlg.GetPath().c_str(), "w");

		for(timeline3_map::iterator p=GetTimeline3Map().begin(); p!=GetTimeline3Map().end(); ++p) {
			fprintf(out, "timeline3_begin(\"%s\")\n", p->first.c_str());

			for(Timeline1::iterator q=p->second->x.begin(); q!=p->second->x.end(); ++q)
				fprintf(out, "timeline3_insert_x(%s)\n", create_point_string(*q));
			for(Timeline1::iterator q=p->second->y.begin(); q!=p->second->y.end(); ++q)
				fprintf(out, "timeline3_insert_y(%s)\n", create_point_string(*q));
			for(Timeline1::iterator q=p->second->z.begin(); q!=p->second->z.end(); ++q)
				fprintf(out, "timeline3_insert_z(%s)\n", create_point_string(*q));

			fprintf(out, "timeline3_end()\n\n");
		}
		fclose(out);
	}
}


void TimelineFrame::OnTextChanged(wxScrollEvent& event)
{
	long frame;
	if( timeline_text->GetValue().ToLong(&frame) ) {
		wxGetApp().current_frame = frame;
		timeline_slider->SetValue( frame );
		RedrawAll();
	}
	else {
		wxString value;
		value.sprintf("%d", wxGetApp().current_frame);
		timeline_text->SetValue( value );
	}
}

void TimelineFrame::OnSliderChanged(wxScrollEvent& event)
{
	wxGetApp().current_frame = event.GetPosition();
	RedrawAll();

	wxString value;
	value.sprintf("%d", wxGetApp().current_frame);
	timeline_text->SetValue( value );
}


void TimelineFrame::OnChoiceSelected(wxCommandEvent&)
{
	int pos = timeline_choice->GetSelection();

	if(pos!=-1) {
		GetCurrentTimeline3Name() = timeline_choice->GetString(pos);
		RedrawAll();
	}
}


void TimelineFrame::OnNewButton(wxCommandEvent& event)
{
	timeline3_map& tl3_map = GetTimeline3Map();

	wxTextEntryDialog dialog(this, wxT("enter timeline name"), wxT("new timeline"));

	if(dialog.ShowModal() == wxID_OK) {
		if(dialog.GetValue().Length()==0)
			return;

		GetCurrentTimeline3Name() = dialog.GetValue();
		timeline3_ptr& p = tl3_map[dialog.GetValue()];
		p.reset(new Timeline3());
		p->x.insert(vector2(0, 50));
		p->x.insert(vector2(100, 50));
		p->y.insert(vector2(0, 0));
		p->y.insert(vector2(100, 0));
		p->z.insert(vector2(0, -50));
		p->z.insert(vector2(100, -50));

		RefreshChoice();
		RedrawAll();
	}
}

void TimelineFrame::OnCopyButton(wxCommandEvent& event)
{
	timeline3_map& tl3_map = GetTimeline3Map();

	if(tl3_map.empty()) {
		wxMessageDialog dialog(this, wxT("no timeline exist"), wxT("copy timeline"), wxOK | wxCENTRE);
		dialog.ShowModal();
		return;
	}

	wxTextEntryDialog dialog(this, wxT("enter timeline name"), wxT("copy timeline"));

	if(dialog.ShowModal() == wxID_OK) {
		tl3_map[dialog.GetValue()] = timeline3_ptr( new Timeline3( *tl3_map[GetCurrentTimeline3Name()] ) );
		GetCurrentTimeline3Name() = dialog.GetValue();

		RefreshChoice();
		RedrawAll();
	}
}

void TimelineFrame::OnDeleteButton(wxCommandEvent& event)
{
	timeline3_map& tl3_map = GetTimeline3Map();

	if(tl3_map.empty()) {
		wxMessageDialog dialog(this, wxT("no timeline exist"), wxT("delete timeline"), wxOK | wxCENTRE);
		dialog.ShowModal();
		return;
	}

	wxMessageDialog dialog(this, wxT("delete ") + GetCurrentTimeline3Name() + wxT(", are you sure?"), wxT("delete timeline"),
		wxOK | wxCANCEL | wxCENTRE);

	if(dialog.ShowModal() == wxID_OK) {
		tl3_map.erase( tl3_map.find(GetCurrentTimeline3Name()) );

		if(tl3_map.empty())
			GetCurrentTimeline3Name() = wxT("");
		else
			GetCurrentTimeline3Name() = tl3_map.begin()->first;

		RefreshChoice();
		RedrawAll();
	}
}

void TimelineFrame::OnEditModeSelected(wxCommandEvent& event)
{
	static map<wxString, int> table;
	if(table.empty()) {
		table[ wxT("edit") ]   = TimelineCanvas::EM_EDIT;
		table[ wxT("insert") ] = TimelineCanvas::EM_INSERT;
		table[ wxT("delete") ] = TimelineCanvas::EM_DELETE;
	}

	canvas->edit_mode = table[mode_selector->GetStringSelection()];
}


void TimelineFrame::RefreshChoice()
{
	timeline3_map& tl3_map = GetTimeline3Map();

	timeline_choice->Clear();
	for(timeline3_map::iterator p=tl3_map.begin(); p!=tl3_map.end(); ++p)
		timeline_choice->Append(p->first);

	if(timeline_choice->GetCount()==0) {
		return;
	}

	int result = timeline_choice->FindString( GetCurrentTimeline3Name() );
	if(result!=-1) {
		timeline_choice->SetSelection(result);
	}
	else {
		GetCurrentTimeline3Name() = timeline_choice->GetString(0);
	}
}






BEGIN_EVENT_TABLE( TimelineCanvas, wxGLCanvas )
	EVT_SIZE( TimelineCanvas::OnSize )
	EVT_PAINT( TimelineCanvas::OnPaint )
	EVT_KEY_DOWN( TimelineCanvas::OnKeyDown )
	EVT_KEY_UP( TimelineCanvas::OnKeyUp )
	EVT_MOUSE_EVENTS( TimelineCanvas::OnMouse )
	EVT_ENTER_WINDOW( TimelineCanvas::OnEnterWindow )
END_EVENT_TABLE()

TimelineCanvas::TimelineCanvas(wxWindow *parent, const wxPoint& pos, const wxSize& size) :
	wxGLCanvas(parent, -1, pos, size)
{
	screen_region[0] = screen_region[1] = screen_region[2] = screen_region[3] = 0.0f;
	edit_mode = EM_EDIT;
	toggle_help = true;
	focus_point = Timeline1::iterator();
	focus_tl1 = 0;
}

void TimelineCanvas::AdjustTimeSlider()
{
	if(GetTimeline3Map().empty())
		return;

	Timeline3 &tl3 = GetTimeline3();
	float min_x = min<float>(tl3.x.front().pos.x, min<float>(tl3.y.front().pos.x, tl3.z.front().pos.x));
	float max_x = max<float>(tl3.x.back().pos.x, max<float>(tl3.y.back().pos.x, tl3.z.back().pos.x));

	GetTimelineFrame()->GetTimelineSlider()->SetRange(min_x, max_x);
}


void TimelineCanvas::OnPaint(wxPaintEvent& event)
{
	Render();
}

void TimelineCanvas::OnSize(wxSizeEvent& event)
{
	wxGLCanvas::OnSize(event);

	int w, h;
	GetClientSize(&w, &h);
	screen_region[0] = 0;
	screen_region[1] = w;
	screen_region[2] = -h/2;
	screen_region[3] = h/2;

	SetCurrent();
	glViewport(0, 0, w, h);

	Render();
}

void TimelineCanvas::OnKeyDown(wxKeyEvent& event)
{
	bool redraw_flag = false;

	switch(event.KeyCode()) {
	case 'E':
		edit_mode = EM_EDIT;
		GetTimelineFrame()->GetModeSelector()->SetStringSelection("edit");
		break;
	case 'I':
		edit_mode = EM_INSERT;
		GetTimelineFrame()->GetModeSelector()->SetStringSelection("insert");
		break;
	case 'D':
		edit_mode = EM_DELETE;
		GetTimelineFrame()->GetModeSelector()->SetStringSelection("delete");
		break;
	}

	if(redraw_flag)
		Render();
}

void TimelineCanvas::OnKeyUp(wxKeyEvent& event)
{}







Timeline1* GetNearLine(const vector2& pos, float r=8)
{
	if(GetTimeline3Map().empty())
		return 0;

	Timeline3& tl3 = GetTimeline3();

	if( (vector2(pos.x, tl3.x.getValue(pos.x)) - pos).square() < r*r ) {
		return &tl3.x;
	}
	if( (vector2(pos.x, tl3.y.getValue(pos.x)) - pos).square() < r*r ) {
		return &tl3.y;
	}
	if( (vector2(pos.x, tl3.z.getValue(pos.x)) - pos).square() < r*r ) {
		return &tl3.z;
	}

	return 0;
}

pair<Timeline1*, Timeline1::iterator> GetNearPoint(const vector2& pos, float r=8)
{
	if(GetTimeline3Map().empty())
		return pair<Timeline1*, Timeline1::iterator>(0, Timeline1::iterator());

	Timeline3& tl3 = GetTimeline3();

	for(Timeline1::iterator p=tl3.x.begin(); p!=tl3.x.end(); ++p) {
		if( (p->pos - pos).square() < r*r ) {
			return pair<Timeline1*, Timeline1::iterator>(&tl3.x, p);
		}
	}
	for(Timeline1::iterator p=tl3.y.begin(); p!=tl3.y.end(); ++p) {
		if( (p->pos - pos).square() < r*r ) {
			return pair<Timeline1*, Timeline1::iterator>(&tl3.y, p);
		}
	}
	for(Timeline1::iterator p=tl3.z.begin(); p!=tl3.z.end(); ++p) {
		if( (p->pos - pos).square() < r*r ) {
			return pair<Timeline1*, Timeline1::iterator>(&tl3.z, p);
		}
	}

	return pair<Timeline1*, Timeline1::iterator>(0, Timeline1::iterator());
}



void TimelineCanvas::OnMouse(wxMouseEvent& event)
{
	if(event.IsButton())
		OnMouseButton(event);
	else if(event.Moving())
		OnMouseMove(event);
}

void TimelineCanvas::OnMouseButton(wxMouseEvent& event)
{
	if(GetTimeline3Map().empty())
		return;

	bool redraw_flag = false;

	if(event.ButtonDown()) {
		if(edit_mode==EM_EDIT) {
			pair<Timeline1*, Timeline1::iterator> result = GetNearPoint(mouse_pos);

			if(event.MiddleDown()) {
				focus_tl1 = result.first;
				focus_point = result.second;
				redraw_flag = true;
			}
			else if(event.LeftDown()) {
				if(result.first!=0 && result.second!=Timeline1::iterator()) {
					focus_tl1 = result.first;
					focus_point = result.second;
					redraw_flag = true;
				}
			}
		}

		if(edit_mode==EM_DELETE) {
			pair<Timeline1*, Timeline1::iterator> result = GetNearPoint(mouse_pos);
			if(result.first!=0 && result.second!=Timeline1::iterator()) {
				result.first->erase(result.second);
				redraw_flag = true;
			}
		}

		if(edit_mode==EM_INSERT) {
			Timeline1 *near_line = GetNearLine(mouse_pos);
			if(near_line!=0) {
				focus_point = Timeline1::iterator();
				near_line->insert( vector2(mouse_pos.x, near_line->getValue(mouse_pos.x)) );
				redraw_flag = true;
			}
		}
	}
	else if(event.ButtonUp()) {
		if(edit_mode==EM_EDIT) {
			if(focus_tl1!=0 && focus_point!=Timeline1::iterator()) {
				if(event.LeftDown() && ((focus_point->in - focus_point->pos).square() < 8*8)) {
					focus_point->in = focus_point->pos;
					redraw_flag = true;
				}
				else if(event.RightDown() && ((focus_point->out - focus_point->pos).square() < 8*8)) {
					 focus_point->out = focus_point->pos;
					redraw_flag = true;
				}
			}
		}
	}

	if(redraw_flag)
		Render();
}

void TimelineCanvas::OnMouseMove(wxMouseEvent& event)
{
	int window_width, window_height;
	GetClientSize(&window_width, &window_height);

	float x_rate = static_cast<float>(event.GetX())/static_cast<float>(window_width);
	float y_rate = static_cast<float>(event.GetY())/static_cast<float>(window_height);
	mouse_pos.x = screen_region[0]+(screen_region[1]-screen_region[0])*x_rate;
	mouse_pos.y = screen_region[3]-(screen_region[3]-screen_region[2])*y_rate;

	wxString status;
	status.sprintf("%.2f, %.2f", mouse_pos.x, mouse_pos.y);
	GetTimelineFrame()->SetStatusText( status );


	if(GetTimeline3Map().empty())
		return;

	bool redraw_flag = false;
	vector2 rel;
	rel.x = event.GetX() - pre_mouse_pos.x;
	rel.y = -(event.GetY() - pre_mouse_pos.y);

	if(focus_point!=Timeline1::iterator()) {
		bool resample_flag=false;

		if(event.MiddleIsDown()) {
			focus_point->pos += rel;
			focus_point->in  += rel;
			focus_point->out += rel;
			resample_flag = true;
		}
		else if(event.LeftIsDown()) {
			focus_point->in += rel;
			resample_flag = true;
		}
		else if(event.RightIsDown()) {
			focus_point->out += rel;
			resample_flag = true;
		}

		if(resample_flag) {
			GetTimeline3().sampling();
			redraw_flag = true;
		}

		AdjustTimeSlider();
	}
	else if(event.MiddleIsDown()) {
		float move_x = rel.x;
		float move_y = rel.y;
		screen_region[0] -= move_x;
		screen_region[1] -= move_x;
		screen_region[2] -= move_y;
		screen_region[3] -= move_y;
		redraw_flag = true;
	}
	else if(event.RightIsDown()) {
		vector2 center((screen_region[1]-screen_region[0])/2, (screen_region[3]-screen_region[2])/2);
		float scale_rate = (rel.x+rel.y)*0.005;
		screen_region[0] = screen_region[0]+((center.x-screen_region[0])*scale_rate);
		screen_region[1] = screen_region[1]+((center.x-screen_region[1])*scale_rate);
		screen_region[2] = screen_region[2]+((center.y-screen_region[2])*scale_rate);
		screen_region[3] = screen_region[3]+((center.y-screen_region[3])*scale_rate);
		redraw_flag = true;
	}

	pre_mouse_pos.x = event.GetX();
	pre_mouse_pos.y = event.GetY();

	if(redraw_flag)
		Render();
}


void TimelineCanvas::OnEnterWindow( wxMouseEvent& event )
{
	SetFocus();
}






template<class Iter>
void draw_points(Iter begin, Iter end)
{
	glBegin(GL_POINTS);
	for(; begin!=end; ++begin) {
		glVertex2fv( begin->pos.v );
	}
	glEnd();
}

template<class Iter>
void draw_lines(Iter begin, Iter end)
{
	glBegin(GL_LINE_STRIP);
	for(; begin!=end; ++begin) {
		glVertex2fv( begin->v );
	}
	glEnd();
}

void draw_detail(Timeline1::point_t& p)
{
	glBegin(GL_POINTS);
	glVertex2fv( p.pos.v );
	glVertex2fv( p.in.v );
	glVertex2fv( p.out.v );
	glEnd();
	glBegin(GL_LINE_STRIP);
	glVertex2fv( p.in.v );
	glVertex2fv( p.pos.v );
	glVertex2fv( p.out.v );
	glEnd();
}





void TimelineCanvas::Render()
{
	wxPaintDC dc(this);
	SetCurrent();


	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(screen_region[0], screen_region[1], screen_region[2], screen_region[3], 1, 50000);
	glMatrixMode( GL_MODELVIEW );

	gl::Camera camera;
	camera.position=vector4(0, 0, 100);
	camera.target=vector4(0, 0, 0);
	camera.look();

	glColor4ub(255, 255, 255, 255);
	glBegin(GL_LINE_STRIP);
	glVertex2f( GetCurrentFrame(), screen_region[2] );
	glVertex2f( GetCurrentFrame(), screen_region[3] );
	glEnd();

	if(!GetTimeline3Map().empty()) {
		Timeline3& tl3 = GetTimeline3();

		glColor4ub(255, 0, 0, 255);
		draw_lines( tl3.x.s_points.begin(), tl3.x.s_points.end() );

		glColor4ub(0, 255, 0, 255);
		draw_lines( tl3.y.s_points.begin(), tl3.y.s_points.end() );

		glColor4ub(0, 0, 255, 255);
		draw_lines( tl3.z.s_points.begin(), tl3.z.s_points.end() );

		glPointSize(7);
		glColor4ub(128, 128, 128, 255);
		draw_points( tl3.x.begin(), tl3.x.end() );
		draw_points( tl3.y.begin(), tl3.y.end() );
		draw_points( tl3.z.begin(), tl3.z.end() );

		if(focus_point!=Timeline1::iterator()) {
			glPointSize(10);
			glColor4ub(255, 255, 255, 255);
			glBegin(GL_POINTS);
			draw_detail( *focus_point );
		}
	}

	SwapBuffers();
}
