#ifndef IST_COLLISION_DETECTOR
#define IST_COLLISION_DETECTOR

#include <vector>
#include "matrix.h"
#include "base_polygon_model.h"


namespace ist {

namespace collision_type {
	enum {
		BASE,
		POINT,
		LINE,
		POLYGON
	};
}

class BaseCollisionModel
{
public:
	virtual ~BaseCollisionModel() {}
	virtual int type() const { return collision_type::BASE; }
	virtual void transform(const matrix44&) {}
	const BoundingBox& getBoundingBox() { return bb; }

protected:
	BoundingBox bb;
};


class PointCollisionModel : public BaseCollisionModel
{
friend struct CollisionDetector;
public:
	vector4 pos;
	float radius;

	PointCollisionModel() : radius(0) {}
	int type() const { return collision_type::POINT; }
	void transform(const matrix44 &trans);

private:
	vector4 t_pos;
};


class LineCollisionModel : public BaseCollisionModel
{
friend struct CollisionDetector;
public:
	vector4 head, tail;
	float radius;

	LineCollisionModel() : radius(0) {}
	int type() const { return collision_type::LINE; }
	void transform(const matrix44 &trans);

private:
	vector4 t_head, t_tail;
};


struct PolygonCollisionModel : public BaseCollisionModel
{
friend struct CollisionDetector;
public:
	PolygonCollisionModel() : chunc(0) {}
	int type() const { return collision_type::POLYGON; }

	void transform(const matrix44 &trans);
	void setChunc(const BasePolygonChunc *_chunc) {
		if(_chunc==0) return;

		chunc=_chunc;
		t_vertex.resize(chunc->getVertexNum());
		t_normal.resize(chunc->getFaceNum());
	}

	const BasePolygonChunc* getChunc() const { return chunc; }

private:
	const BasePolygonChunc *chunc;
	vector4 center;
	std::vector<vector4> t_vertex;
	std::vector<vector4> t_normal;
};



struct CollisionDetector
{
	bool result;
	float distance;
	vector4 collide_pos, collide_normal;

	CollisionDetector();
	bool detect(const BaseCollisionModel &base, const BaseCollisionModel &target);

	template <class T, class U>
	bool detect_(const T &base, const U &target);
};


} //ist

#endif
